# Form implementation generated from reading ui file 'fitsettings.ui'
#
# Created by: PyQt6 UI code generator 6.7.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_fitparbox(object):
    def setupUi(self, fitparbox):
        fitparbox.setObjectName("fitparbox")
        fitparbox.resize(582, 447)
        self.gridLayout_2 = QtWidgets.QGridLayout(fitparbox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.abundcutlabel = QtWidgets.QLabel(parent=fitparbox)
        self.abundcutlabel.setObjectName("abundcutlabel")
        self.gridLayout_2.addWidget(self.abundcutlabel, 6, 0, 1, 1)
        self.line = QtWidgets.QFrame(parent=fitparbox)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 2, 0, 1, 2)
        self.wavecutlabel = QtWidgets.QLabel(parent=fitparbox)
        self.wavecutlabel.setObjectName("wavecutlabel")
        self.gridLayout_2.addWidget(self.wavecutlabel, 4, 0, 1, 1)
        self.convovcutlabel = QtWidgets.QLabel(parent=fitparbox)
        self.convovcutlabel.setObjectName("convovcutlabel")
        self.gridLayout_2.addWidget(self.convovcutlabel, 5, 0, 1, 1)
        self.abunditerlabel = QtWidgets.QLabel(parent=fitparbox)
        self.abunditerlabel.setObjectName("abunditerlabel")
        self.gridLayout_2.addWidget(self.abunditerlabel, 12, 0, 1, 1)
        self.wavecutvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.wavecutvalue.setMaximum(999.99)
        self.wavecutvalue.setSingleStep(0.01)
        self.wavecutvalue.setObjectName("wavecutvalue")
        self.gridLayout_2.addWidget(self.wavecutvalue, 4, 1, 1, 1)
        self.repfitvalue = QtWidgets.QSpinBox(parent=fitparbox)
        self.repfitvalue.setMaximum(10)
        self.repfitvalue.setObjectName("repfitvalue")
        self.gridLayout_2.addWidget(self.repfitvalue, 1, 1, 1, 1)
        self.contitervalue = QtWidgets.QSpinBox(parent=fitparbox)
        self.contitervalue.setMaximum(10000)
        self.contitervalue.setObjectName("contitervalue")
        self.gridLayout_2.addWidget(self.contitervalue, 10, 1, 1, 1)
        self.contiterlabel = QtWidgets.QLabel(parent=fitparbox)
        self.contiterlabel.setObjectName("contiterlabel")
        self.gridLayout_2.addWidget(self.contiterlabel, 10, 0, 1, 1)
        self.plotcutlabel = QtWidgets.QLabel(parent=fitparbox)
        self.plotcutlabel.setObjectName("plotcutlabel")
        self.gridLayout_2.addWidget(self.plotcutlabel, 7, 0, 1, 1)
        self.line_2 = QtWidgets.QFrame(parent=fitparbox)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 8, 0, 1, 2)
        self.waveconviterlabel = QtWidgets.QLabel(parent=fitparbox)
        self.waveconviterlabel.setObjectName("waveconviterlabel")
        self.gridLayout_2.addWidget(self.waveconviterlabel, 11, 0, 1, 1)
        self.abundcutvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.abundcutvalue.setMaximum(999.99)
        self.abundcutvalue.setSingleStep(0.01)
        self.abundcutvalue.setObjectName("abundcutvalue")
        self.gridLayout_2.addWidget(self.abundcutvalue, 6, 1, 1, 1)
        self.cutvaltitle = QtWidgets.QLabel(parent=fitparbox)
        self.cutvaltitle.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.cutvaltitle.setObjectName("cutvaltitle")
        self.gridLayout_2.addWidget(self.cutvaltitle, 3, 0, 1, 2)
        self.abunditervalue = QtWidgets.QSpinBox(parent=fitparbox)
        self.abunditervalue.setMaximum(10000)
        self.abunditervalue.setObjectName("abunditervalue")
        self.gridLayout_2.addWidget(self.abunditervalue, 12, 1, 1, 1)
        self.plotcutvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.plotcutvalue.setMaximum(999.99)
        self.plotcutvalue.setSingleStep(0.01)
        self.plotcutvalue.setObjectName("plotcutvalue")
        self.gridLayout_2.addWidget(self.plotcutvalue, 7, 1, 1, 1)
        self.fitmaxiterlabel = QtWidgets.QLabel(parent=fitparbox)
        self.fitmaxiterlabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.fitmaxiterlabel.setObjectName("fitmaxiterlabel")
        self.gridLayout_2.addWidget(self.fitmaxiterlabel, 9, 0, 1, 2)
        self.convovcutvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.convovcutvalue.setMaximum(999.99)
        self.convovcutvalue.setSingleStep(0.01)
        self.convovcutvalue.setObjectName("convovcutvalue")
        self.gridLayout_2.addWidget(self.convovcutvalue, 5, 1, 1, 1)
        self.waveconvitervalue = QtWidgets.QSpinBox(parent=fitparbox)
        self.waveconvitervalue.setMaximum(10000)
        self.waveconvitervalue.setObjectName("waveconvitervalue")
        self.gridLayout_2.addWidget(self.waveconvitervalue, 11, 1, 1, 1)
        self.repfitlabel = QtWidgets.QLabel(parent=fitparbox)
        self.repfitlabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.repfitlabel.setObjectName("repfitlabel")
        self.gridLayout_2.addWidget(self.repfitlabel, 0, 0, 1, 2)
        self.waveboundlabel = QtWidgets.QLabel(parent=fitparbox)
        self.waveboundlabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.waveboundlabel.setObjectName("waveboundlabel")
        self.gridLayout_2.addWidget(self.waveboundlabel, 0, 3, 1, 2)
        self.waveboundmaxshiftlabel = QtWidgets.QLabel(parent=fitparbox)
        self.waveboundmaxshiftlabel.setObjectName("waveboundmaxshiftlabel")
        self.gridLayout_2.addWidget(self.waveboundmaxshiftlabel, 1, 3, 1, 1)
        self.waveboundmaxshiftvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.waveboundmaxshiftvalue.setObjectName("waveboundmaxshiftvalue")
        self.gridLayout_2.addWidget(self.waveboundmaxshiftvalue, 1, 4, 1, 1)
        self.line_3 = QtWidgets.QFrame(parent=fitparbox)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout_2.addWidget(self.line_3, 2, 3, 1, 2)
        self.convboundlabel = QtWidgets.QLabel(parent=fitparbox)
        self.convboundlabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.convboundlabel.setObjectName("convboundlabel")
        self.gridLayout_2.addWidget(self.convboundlabel, 3, 3, 1, 2)
        self.convboundminlabel = QtWidgets.QLabel(parent=fitparbox)
        self.convboundminlabel.setObjectName("convboundminlabel")
        self.gridLayout_2.addWidget(self.convboundminlabel, 4, 3, 1, 1)
        self.convboundminvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.convboundminvalue.setObjectName("convboundminvalue")
        self.gridLayout_2.addWidget(self.convboundminvalue, 4, 4, 1, 1)
        self.convboundmaxlabel = QtWidgets.QLabel(parent=fitparbox)
        self.convboundmaxlabel.setObjectName("convboundmaxlabel")
        self.gridLayout_2.addWidget(self.convboundmaxlabel, 5, 3, 1, 1)
        self.convboundmaxvalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.convboundmaxvalue.setObjectName("convboundmaxvalue")
        self.gridLayout_2.addWidget(self.convboundmaxvalue, 5, 4, 1, 1)
        self.repfitvaluelabel = QtWidgets.QLabel(parent=fitparbox)
        self.repfitvaluelabel.setObjectName("repfitvaluelabel")
        self.gridLayout_2.addWidget(self.repfitvaluelabel, 1, 0, 1, 1)
        self.line_4 = QtWidgets.QFrame(parent=fitparbox)
        self.line_4.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_4.setObjectName("line_4")
        self.gridLayout_2.addWidget(self.line_4, 6, 3, 1, 2)
        self.contfitparamlabel = QtWidgets.QLabel(parent=fitparbox)
        self.contfitparamlabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.contfitparamlabel.setObjectName("contfitparamlabel")
        self.gridLayout_2.addWidget(self.contfitparamlabel, 7, 3, 1, 2)
        self.contfitparalphalabel = QtWidgets.QLabel(parent=fitparbox)
        self.contfitparalphalabel.setObjectName("contfitparalphalabel")
        self.gridLayout_2.addWidget(self.contfitparalphalabel, 8, 3, 1, 1)
        self.contfitparalphavalue = QtWidgets.QDoubleSpinBox(parent=fitparbox)
        self.contfitparalphavalue.setObjectName("contfitparalphavalue")
        self.gridLayout_2.addWidget(self.contfitparalphavalue, 8, 4, 1, 1)
        self.contfitparepslabel = QtWidgets.QLabel(parent=fitparbox)
        self.contfitparepslabel.setObjectName("contfitparepslabel")
        self.gridLayout_2.addWidget(self.contfitparepslabel, 9, 3, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(parent=fitparbox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.contfitparepsvalue = QtWidgets.QSpinBox(parent=fitparbox)
        self.contfitparepsvalue.setObjectName("contfitparepsvalue")
        self.horizontalLayout.addWidget(self.contfitparepsvalue)
        self.label_2 = QtWidgets.QLabel(parent=fitparbox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.gridLayout_2.addLayout(self.horizontalLayout, 9, 4, 1, 1)
        self.line_5 = QtWidgets.QFrame(parent=fitparbox)
        self.line_5.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_5.setObjectName("line_5")
        self.gridLayout_2.addWidget(self.line_5, 0, 2, 13, 1)
        self.okcancelbutton = QtWidgets.QDialogButtonBox(parent=fitparbox)
        self.okcancelbutton.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.okcancelbutton.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.okcancelbutton.setObjectName("okcancelbutton")
        self.gridLayout_2.addWidget(self.okcancelbutton, 12, 3, 1, 2)

        self.retranslateUi(fitparbox)
        self.okcancelbutton.accepted.connect(fitparbox.accept) # type: ignore
        self.okcancelbutton.rejected.connect(fitparbox.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(fitparbox)

    def retranslateUi(self, fitparbox):
        _translate = QtCore.QCoreApplication.translate
        fitparbox.setWindowTitle(_translate("fitparbox", "Fit Settings"))
        self.abundcutlabel.setText(_translate("fitparbox", "Abundance"))
        self.wavecutlabel.setToolTip(_translate("fitparbox", "Continuum range is not used in \"Eq. Width\" method, but Wave. Shift is."))
        self.wavecutlabel.setText(_translate("fitparbox", "Cont/Wave. Shift"))
        self.convovcutlabel.setToolTip(_translate("fitparbox", "Can not be higher than the value of Continuum/Wave. Shift."))
        self.convovcutlabel.setText(_translate("fitparbox", "Convolution"))
        self.abunditerlabel.setText(_translate("fitparbox", "Abundance"))
        self.wavecutvalue.setToolTip(_translate("fitparbox", "Continuum range is not used in \"Eq. Width\" method, but Wave. Shift is."))
        self.contiterlabel.setText(_translate("fitparbox", "Continuum"))
        self.plotcutlabel.setText(_translate("fitparbox", "Plot"))
        self.waveconviterlabel.setText(_translate("fitparbox", "Wave. Shift/Conv."))
        self.cutvaltitle.setToolTip(_translate("fitparbox", "The range that the fit will be applied. The value represents the full range (half of the value left to the fitted line and half to the right)."))
        self.cutvaltitle.setText(_translate("fitparbox", "Interaction range (Angstrom) for each fit"))
        self.fitmaxiterlabel.setText(_translate("fitparbox", "Fit Maximum Iterations"))
        self.convovcutvalue.setToolTip(_translate("fitparbox", "Can not be higher than the value of Continuum/Wave. Shift."))
        self.repfitlabel.setToolTip(_translate("fitparbox", "The number of times that it will try to fit each line. The subsequent iteration uses the previous result as start point. Default is 2."))
        self.repfitlabel.setText(_translate("fitparbox", "Forced Loop Iterations"))
        self.waveboundlabel.setText(_translate("fitparbox", "Wave. Shift (Angstrom) Boundaries"))
        self.waveboundmaxshiftlabel.setText(_translate("fitparbox", "Max. Shift (each side)"))
        self.convboundlabel.setText(_translate("fitparbox", "Convolution Boundaries"))
        self.convboundminlabel.setText(_translate("fitparbox", "Minimum"))
        self.convboundmaxlabel.setText(_translate("fitparbox", "Maximum"))
        self.repfitvaluelabel.setText(_translate("fitparbox", "Value"))
        self.contfitparamlabel.setText(_translate("fitparbox", "Continuum Fit Parameters"))
        self.contfitparalphalabel.setText(_translate("fitparbox", "Tolerance (alpha)"))
        self.contfitparepslabel.setText(_translate("fitparbox", "Error (epsilon)"))
        self.label.setText(_translate("fitparbox", "10 ^ ( -"))
        self.label_2.setText(_translate("fitparbox", ")"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    fitparbox = QtWidgets.QDialog()
    ui = Ui_fitparbox()
    ui.setupUi(fitparbox)
    fitparbox.show()
    sys.exit(app.exec())
